@@ -206,7 +206,11 @@ pub fn float_rot(push_state: &mut PushState, _instruction_cache: &InstructionCac
 /// INTEGER.
 pub fn float_shove(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(shove_index) = push_state.int_stack.pop() {
-        push_state.float_stack.shove(shove_index as usize);
+        let corr_index = i32::max(
+            i32::min((push_state.float_stack.size() as i32) - 1, shove_index),
+            0,
+        ) as usize;
+        push_state.float_stack.shove(corr_index as usize);
     }
 }
 
@@ -239,16 +243,24 @@ pub fn float_tan(push_state: &mut PushState, _instruction_cache: &InstructionCac
 /// FLOAT.YANK: Removes an indexed item from "deep" in the stack and pushes it on top of the stack.
 /// The index is taken from the INTEGER stack.
 pub fn float_yank(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
-    if let Some(idx) = push_state.int_stack.pop() {
-        push_state.float_stack.yank(idx as usize);
+    if let Some(index) = push_state.int_stack.pop() {
+        let corr_index = i32::max(
+            i32::min((push_state.float_stack.size() as i32) - 1, index),
+            0,
+        ) as usize;
+        push_state.float_stack.yank(corr_index as usize);
     }
 }
 
 /// FLOAT.YANKDUP: Pushes a copy of an indexed item "deep" in the stack onto the top of the stack,
 /// without removing the deep item. The index is taken from the INTEGER stack.
 pub fn float_yank_dup(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
-    if let Some(idx) = push_state.int_stack.pop() {
-        if let Some(deep_item) = push_state.float_stack.copy(idx as usize) {
+    if let Some(index) = push_state.int_stack.pop() {
+        let corr_index = i32::max(
+            i32::min((push_state.float_stack.size() as i32) - 1, index),
+            0,
+        ) as usize;
+        if let Some(deep_item) = push_state.float_stack.copy(corr_index as usize) {
             push_state.float_stack.push(deep_item);
         }
     }
