@@ -208,7 +208,11 @@ pub fn integer_rot(push_state: &mut PushState, _instruction_cache: &InstructionC
 /// top INTEGER. The index position is calculated after the index is removed.
 pub fn integer_shove(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(shove_index) = push_state.int_stack.pop() {
-        push_state.int_stack.shove(shove_index as usize);
+        let corr_index = i32::max(
+            i32::min((push_state.int_stack.size() as i32) - 1, shove_index),
+            0,
+        ) as usize;
+        push_state.int_stack.shove(corr_index as usize);
     }
 }
 
@@ -228,8 +232,10 @@ pub fn integer_swap(push_state: &mut PushState, _instruction_cache: &Instruction
 /// stack. The index is taken from the INTEGER stack, and the indexing is done after the index is
 /// removed.
 pub fn integer_yank(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
-    if let Some(idx) = push_state.int_stack.pop() {
-        push_state.int_stack.yank(idx as usize);
+    if let Some(index) = push_state.int_stack.pop() {
+        let corr_index =
+            i32::max(i32::min((push_state.int_stack.size() as i32) - 1, index), 0) as usize;
+        push_state.int_stack.yank(corr_index as usize);
     }
 }
 
@@ -237,8 +243,10 @@ pub fn integer_yank(push_state: &mut PushState, _instruction_cache: &Instruction
 /// stack, without removing the deep item. The index is taken from the INTEGER stack, and the
 /// indexing is done after the index is removed.
 pub fn integer_yank_dup(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
-    if let Some(idx) = push_state.int_stack.pop() {
-        if let Some(deep_item) = push_state.int_stack.copy(idx as usize) {
+    if let Some(index) = push_state.int_stack.pop() {
+        let corr_index =
+            i32::max(i32::min((push_state.int_stack.size() as i32) - 1, index), 0) as usize;
+        if let Some(deep_item) = push_state.int_stack.copy(corr_index as usize) {
             push_state.int_stack.push(deep_item);
         }
     }
