@@ -148,6 +148,10 @@ pub fn load_vector_instructions(map: &mut HashMap<String, Instruction>) {
         String::from("BOOLVECTOR.ONES"),
         Instruction::new(bool_vector_ones),
     );
+    map.insert(
+        String::from("BOOLVECTOR.POP"),
+        Instruction::new(bool_vector_pop),
+    );
     map.insert(
         String::from("BOOLVECTOR.RAND"),
         Instruction::new(bool_vector_rand),
@@ -226,6 +230,10 @@ pub fn load_vector_instructions(map: &mut HashMap<String, Instruction>) {
         String::from("INTVECTOR.ONES"),
         Instruction::new(int_vector_ones),
     );
+    map.insert(
+        String::from("INTVECTOR.POP"),
+        Instruction::new(int_vector_pop),
+    );
     map.insert(
         String::from("INTVECTOR.RAND"),
         Instruction::new(int_vector_rand),
@@ -299,6 +307,10 @@ pub fn load_vector_instructions(map: &mut HashMap<String, Instruction>) {
         String::from("FLOATVECTOR.ONES"),
         Instruction::new(float_vector_ones),
     );
+    map.insert(
+        String::from("FLOATVECTOR.POP"),
+        Instruction::new(float_vector_pop),
+    );
     map.insert(
         String::from("FLOATVECTOR.RAND"),
         Instruction::new(float_vector_rand),
@@ -342,7 +354,7 @@ pub fn bool_vector_set(push_state: &mut PushState, _instruction_cache: &Instruct
         if let Some(new_element) = push_state.bool_stack.pop() {
             if let Some(item_to_change) = push_state.bool_vector_stack.get_mut(0) {
                 let i =
-                    i32::min(i32::max(index, 0), item_to_change.values.len() as i32 - 1) as usize;
+                    i32::max(i32::min(index, item_to_change.values.len() as i32 - 1), 0) as usize;
                 item_to_change.values[i] = new_element;
             }
         }
@@ -376,7 +388,7 @@ pub fn bool_vector_and(push_state: &mut PushState, _instruction_cache: &Instruct
 pub fn bool_vector_get(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(index) = push_state.int_stack.pop() {
         if let Some(element) = push_state.bool_vector_stack.get(0) {
-            let i = i32::min(i32::max(index, 0), element.values.len() as i32 - 1) as usize;
+            let i = i32::max(i32::min(index, element.values.len() as i32 - 1), 0) as usize;
             push_state.bool_stack.push(element.values[i].clone());
         }
     }
@@ -473,6 +485,11 @@ pub fn bool_vector_ones(push_state: &mut PushState, _instruction_cache: &Instruc
     }
 }
 
+/// BOOLVECTOR.POP: Pops the BOOLVECTOR stack.
+pub fn bool_vector_pop(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
+    push_state.bool_vector_stack.pop();
+}
+
 /// BOOLVECTOR.RAND: Pushes a newly generated random BOOLVECTOR. The size is taken from the INTEGER
 /// stack, the sparsity from the FLOAT stack. If the size is <0 or the sparcity not in [0,1] this
 /// acts as a NOOP.
@@ -490,7 +507,14 @@ pub fn bool_vector_rand(push_state: &mut PushState, _instruction_cache: &Instruc
 /// top INTEGER. The index position is calculated after the index is removed.
 pub fn bool_vector_shove(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(shove_index) = push_state.int_stack.pop() {
-        push_state.bool_vector_stack.shove(shove_index as usize);
+        let corr_index = i32::max(
+            i32::min(
+                (push_state.bool_vector_stack.size() as i32) - 1,
+                shove_index,
+            ),
+            0,
+        ) as usize;
+        push_state.bool_vector_stack.shove(corr_index);
     }
 }
 
@@ -511,7 +535,11 @@ pub fn bool_vector_swap(push_state: &mut PushState, _instruction_cache: &Instruc
 /// removed.
 pub fn bool_vector_yank(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(idx) = push_state.int_stack.pop() {
-        push_state.bool_vector_stack.yank(idx as usize);
+        let corr_index = i32::max(
+            i32::min((push_state.bool_vector_stack.size() as i32) - 1, idx),
+            0,
+        ) as usize;
+        push_state.bool_vector_stack.yank(corr_index as usize);
     }
 }
 
@@ -520,7 +548,11 @@ pub fn bool_vector_yank(push_state: &mut PushState, _instruction_cache: &Instruc
 /// indexing is done after the index is removed.
 pub fn bool_vector_yank_dup(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(idx) = push_state.int_stack.pop() {
-        if let Some(deep_item) = push_state.bool_vector_stack.copy(idx as usize) {
+        let corr_index = i32::max(
+            i32::min((push_state.bool_vector_stack.size() as i32) - 1, idx),
+            0,
+        ) as usize;
+        if let Some(deep_item) = push_state.bool_vector_stack.copy(corr_index as usize) {
             push_state.bool_vector_stack.push(deep_item);
         }
     }
@@ -562,7 +594,7 @@ pub fn int_vector_bool_index(push_state: &mut PushState, _instruction_cache: &In
 pub fn int_vector_get(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(index) = push_state.int_stack.pop() {
         if let Some(element) = push_state.int_vector_stack.get(0) {
-            let i = i32::min(i32::max(index, 0), element.values.len() as i32 - 1) as usize;
+            let i = i32::max(i32::min(index, element.values.len() as i32 - 1), 0) as usize;
             push_state.int_stack.push(element.values[i].clone());
         }
     }
@@ -575,7 +607,7 @@ pub fn int_vector_set(push_state: &mut PushState, _instruction_cache: &Instructi
         if let Some(new_element) = push_state.int_stack.pop() {
             if let Some(item_to_change) = push_state.int_vector_stack.get_mut(0) {
                 let i =
-                    i32::min(i32::max(index, 0), item_to_change.values.len() as i32 - 1) as usize;
+                    i32::max(i32::min(index, item_to_change.values.len() as i32 - 1), 0) as usize;
                 item_to_change.values[i] = new_element;
             }
         }
@@ -721,6 +753,11 @@ pub fn int_vector_ones(push_state: &mut PushState, _instruction_cache: &Instruct
     }
 }
 
+/// INTVECTOR.POP: Pops the INTVECTOR stack.
+pub fn int_vector_pop(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
+    push_state.int_vector_stack.pop();
+}
+
 /// INTVECTOR.RAND: Pushes a newly generated random INTVECTOR. The size, min and max values
 /// taken from the INTEGER stack in that order. If the size is <0 or max < min this act as a NOOP.
 pub fn int_vector_rand(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
@@ -738,7 +775,11 @@ pub fn int_vector_rand(push_state: &mut PushState, _instruction_cache: &Instruct
 /// top INTEGER. The index position is calculated after the index is removed.
 pub fn int_vector_shove(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(shove_index) = push_state.int_stack.pop() {
-        push_state.int_vector_stack.shove(shove_index as usize);
+        let corr_index = i32::max(
+            i32::min((push_state.int_vector_stack.size() as i32) - 1, shove_index),
+            0,
+        ) as usize;
+        push_state.int_vector_stack.shove(corr_index as usize);
     }
 }
 
@@ -759,7 +800,11 @@ pub fn int_vector_swap(push_state: &mut PushState, _instruction_cache: &Instruct
 /// removed.
 pub fn int_vector_yank(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(idx) = push_state.int_stack.pop() {
-        push_state.int_vector_stack.yank(idx as usize);
+        let corr_index = i32::max(
+            i32::min((push_state.int_vector_stack.size() as i32) - 1, idx),
+            0,
+        ) as usize;
+        push_state.int_vector_stack.yank(corr_index as usize);
     }
 }
 
@@ -768,7 +813,11 @@ pub fn int_vector_yank(push_state: &mut PushState, _instruction_cache: &Instruct
 /// indexing is done after the index is removed.
 pub fn int_vector_yank_dup(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(idx) = push_state.int_stack.pop() {
-        if let Some(deep_item) = push_state.int_vector_stack.copy(idx as usize) {
+        let corr_index = i32::max(
+            i32::min((push_state.int_vector_stack.size() as i32) - 1, idx),
+            0,
+        ) as usize;
+        if let Some(deep_item) = push_state.int_vector_stack.copy(corr_index as usize) {
             push_state.int_vector_stack.push(deep_item);
         }
     }
@@ -793,7 +842,7 @@ pub fn int_vector_zeros(push_state: &mut PushState, _instruction_cache: &Instruc
 pub fn float_vector_get(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(index) = push_state.int_stack.pop() {
         if let Some(element) = push_state.float_vector_stack.get(0) {
-            let i = i32::min(i32::max(index, 0), element.values.len() as i32 - 1) as usize;
+            let i = i32::max(i32::min(index, element.values.len() as i32 - 1), 0) as usize;
             push_state.float_stack.push(element.values[i].clone());
         }
     }
@@ -806,7 +855,7 @@ pub fn float_vector_set(push_state: &mut PushState, _instruction_cache: &Instruc
         if let Some(new_element) = push_state.float_stack.pop() {
             if let Some(item_to_change) = push_state.float_vector_stack.get_mut(0) {
                 let i =
-                    i32::min(i32::max(index, 0), item_to_change.values.len() as i32 - 1) as usize;
+                    i32::max(i32::min(index, item_to_change.values.len() as i32 - 1), 0) as usize;
                 item_to_change.values[i] = new_element;
             }
         }
@@ -952,6 +1001,11 @@ pub fn float_vector_ones(push_state: &mut PushState, _instruction_cache: &Instru
     }
 }
 
+/// FLOATVECTOR.POP: Pops the FLOATVECTOR stack.
+pub fn float_vector_pop(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
+    push_state.float_vector_stack.pop();
+}
+
 /// FLOATVECTOR.RAND: Pushes a newly generated random INTVECTOR. The size is taken from the
 /// INTEGER stack while the parameters for mean and standard deviation are the first (top) and
 /// second item on the FLOAT stack. If size < 0 or standard deviation < 0 this act as a NOOP.
@@ -995,7 +1049,14 @@ pub fn float_vector_sine(push_state: &mut PushState, _instruction_cache: &Instru
 /// top INTEGER. The index position is calculated after the index is removed.
 pub fn float_vector_shove(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(shove_index) = push_state.int_stack.pop() {
-        push_state.float_vector_stack.shove(shove_index as usize);
+        let corr_index = i32::max(
+            i32::min(
+                (push_state.float_vector_stack.size() as i32) - 1,
+                shove_index,
+            ),
+            0,
+        ) as usize;
+        push_state.float_vector_stack.shove(corr_index as usize);
     }
 }
 
@@ -1016,7 +1077,11 @@ pub fn float_vector_swap(push_state: &mut PushState, _instruction_cache: &Instru
 /// removed.
 pub fn float_vector_yank(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(idx) = push_state.int_stack.pop() {
-        push_state.float_vector_stack.yank(idx as usize);
+        let corr_index = i32::max(
+            i32::min((push_state.float_vector_stack.size() as i32) - 1, idx),
+            0,
+        ) as usize;
+        push_state.float_vector_stack.yank(corr_index as usize);
     }
 }
 
@@ -1025,7 +1090,11 @@ pub fn float_vector_yank(push_state: &mut PushState, _instruction_cache: &Instru
 /// indexing is done after the index is removed.
 pub fn float_vector_yank_dup(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(idx) = push_state.int_stack.pop() {
-        if let Some(deep_item) = push_state.float_vector_stack.copy(idx as usize) {
+        let corr_index = i32::max(
+            i32::min((push_state.float_vector_stack.size() as i32) - 1, idx),
+            0,
+        ) as usize;
+        if let Some(deep_item) = push_state.float_vector_stack.copy(corr_index as usize) {
             push_state.float_vector_stack.push(deep_item);
         }
     }
@@ -1306,6 +1375,23 @@ mod tests {
             test_state.bool_vector_stack.to_string(),
             "1:[0]; 2:[0]; 3:[1]; 4:[0];"
         );
+        test_state
+            .bool_vector_stack
+            .push(BoolVector::new(vec![true]));
+        test_state.int_stack.push(25);
+        bool_vector_shove(&mut test_state, &icache());
+        assert_eq!(
+            test_state.bool_vector_stack.to_string(),
+            "1:[0]; 2:[0]; 3:[1]; 4:[0]; 5:[1];"
+        );
+        test_state
+            .bool_vector_stack
+            .push(BoolVector::new(vec![true]));
+        test_state.int_stack.push(-2);
+        assert_eq!(
+            test_state.bool_vector_stack.to_string(),
+            "1:[1]; 2:[0]; 3:[0]; 4:[1]; 5:[0]; 6:[1];"
+        );
     }
 
     #[test]
